"""
Auth provide class Auth for authentication account. You can use decorator
auth_interface to create a function with auto generated authentication.
"""
import pili.conf as conf
from urlparse import urlparse
from .utils import send_and_decode, __hmac_sha1__

class Auth(object):
    """
    class Auth store the access_key and secret_key for authentication.
    """
    def __init__(self, access_key, secret_key):
        if not (access_key and secret_key):
            raise ValueError('invalid key')
        self.__access_key, self.__secret_key = access_key, secret_key

    def auth_interface_str(self, raw_str):
        """
        generate sign str.
        """
        encoded = __hmac_sha1__(raw_str, self.__secret_key)
        return 'Qiniu {0}:{1}'.format(self.__access_key, encoded)

def auth_interface(method):
    """
    decorator takes func(**args) return req and change it to
    func(auth, **args) return json result.

    Args:
        func(**args) -> Request

    Returns:
        func(**args) -> dict (decoded json)
    """
    def authed(**args):
        """
        send request and decode response. Return the result in python format.
        """
        req = method(**args)
        parsed = urlparse(req.get_full_url())
        raw_str = '%s %s' % (req.get_method(), parsed.path)
        if parsed.query:
            raw_str += '?%s' % (parsed.query)
        raw_str += '\nHost: %s' % (parsed.netloc)
        if req.has_data():
            raw_str +=  '\nContent-Type: application/json'
        raw_str+="\n\n"
        if req.has_data():
            raw_str += req.get_data()
            req.add_header('Content-Type', 'application/json')
        req.add_header('Authorization', conf.AUTH.auth_interface_str(raw_str))
        return send_and_decode(req)
    return authed




"""
All api for access pili.
More Info: http://pili-io.github.io/docs/v1/index.html
"""
from .auth import auth_interface
import pili.conf as conf
from urllib2 import Request
import json

@auth_interface
def create_stream(**args):
    keyword = ['hub', 'title', 'publishKey', 'publishSecurity']
    if set(args) - set(keyword):
        raise ValueError('invalid key')
    encoded = json.dumps(args)
    url = "http://%s/%s/streams" % (conf.API_HOST, conf.API_VERSION)
    return Request(url=url, data=encoded)

@auth_interface
def get_stream(stream_id):
    url = "http://%s/%s/streams/%s" % (conf.API_HOST, conf.API_VERSION, stream_id)
    return Request(url=url)

@auth_interface
def get_stream_list(**args):
    keyword = ['hub', 'marker', 'limit']
    if set(args) - set(keyword):
        raise ValueError('invalid key')
    url = "http://%s/%s/streams?" % (conf.API_HOST, conf.API_VERSION)
    for k in args:
        if args[k] is not None:
            url += "&%s=%s" % (k, args[k])
    req = Request(url=url)
    return req

@auth_interface
def update_stream(stream_id, **args):
    keyword = ['publishKey', 'publishSecurity']
    if set(args) - set(keyword):
        raise ValueError('invalid key')
    encoded = json.dumps(args)
    url = "http://%s/%s/streams/%s" % (conf.API_HOST, conf.API_VERSION, stream_id)
    return Request(url=url, data=encoded)

@auth_interface
def delete_stream(stream_id):
    url = "http://%s/%s/streams/%s" % (conf.API_HOST, conf.API_VERSION, stream_id)
    req = Request(url=url)
    req.get_method = lambda: 'DELETE'
    return req

@auth_interface
def get_status(stream_id):
    url = "http://%s/%s/streams/%s/status" % (conf.API_HOST, conf.API_VERSION, stream_id)
    return Request(url=url)

@auth_interface
def get_segments(stream_id, start_second, end_second):
    url = "http://%s/%s/streams/%s/segments?start=%s&end=%s" % (conf.API_HOST, conf.API_VERSION, stream_id, start_second, end_second)
    return Request(url=url)



"""
Settings
"""
API_VERSION       = 'v1'
API_HOST          = 'pili.qiniuapi.com'
AUTH              = None
RTMP_PUBLISH_HOST = None
RTMP_PLAY_HOST    = None
HLS_PLAY_HOST     = None
HUB               = None



import pili.api as api
import pili.conf as conf
import time
from .utils import __hmac_sha1__
from urlparse import urlparse

def create_stream(hub=conf.HUB, **args):
    res = api.create_stream(hub=hub, **args)
    return Stream(data=res)

def get_stream(stream_id):
    return Stream(stream_id=stream_id)

def streams(hub=conf.HUB, limit=None):
    marker = None
    while True:
        res = api.get_stream_list(hub=hub, marker=marker, limit=None)
        if res["items"] is not None:
            for data in res["items"]:
                yield Stream(data=data)
        else:
            break
        marker = res["marker"]

class Stream(object):
    """
    Stream is used to control a stream. You should always create a Stream object by the
    interfaces of an application.
    """
    def __init__(self, stream_id=None, data=None):
        if not (stream_id or data):
            raise ValueError('invalid key')
        if not stream_id:
            stream_id = data["id"]
        self.__stream_id__ = stream_id
        self.__data__ = data
        self.play = Play(stream_id)
        self.publish= Publish(stream_id, self.publishSecurity, self.publishKey)

    def __getattr__(self, attr):
        if not self.__data__:
            self.refresh()
        return self.__data__[attr]

    def refresh(self):
        self.__data__ = api.get_stream(stream_id=self.__stream_id__)
        return self.__data__

    def update(self, **args):
        res = api.update_stream(stream_id=self.__stream_id__, **args)
        self.__data__ = res
        return res

    def delete(self):
        res = api.delete_stream(stream_id=self.__stream_id__)
        return res

    def status(self):
        res = api.get_status(stream_id=self.__stream_id__)
        return res

    def get_segments(self, start_second, end_second):
        res = api.get_segments(stream_id=self.__stream_id__, start_second=start_second, end_second=end_second)
        return res


class Play(object):
    """
    Play is used to get the urls for playing the stream
    """
    def __init__(self, stream_id):
        _, self.__hub__, self.__title__ = stream_id.split('.')
        self.rtmp_play_host = conf.RTMP_PLAY_HOST
        self.hls_play_host = conf.HLS_PLAY_HOST
    def __base__(self, protocol, host, profile):
        url = "%s://%s/%s/%s" % (protocol, host, self.__hub__, self.__title__)
        if profile!="":
            url += "@%s" % profile
        return url
    def rtmp_live(self, profile=""):
        return self.__base__("rtmp", self.rtmp_play_host, profile)
    def hls_live(self, profile=""):
        return self.__base__("http", self.hls_play_host, profile)
    def hls_playback(self, start_second, end_second, profile=""):
        url = self.__base__("http", self.hls_play_host, profile)
        url += "?start=%d&end=%d" % (start_second, end_second)
        return url

class Publish(object):
    """
    Publish is used to get the urls for publishing the stream
    """
    def __init__(self, stream_id, security, key):
        _, self.__hub__, self.__title__ = stream_id.split('.')
        self.__security__ = security
        self.__key__ = str(key)
        self.rtmp_publish_host = conf.RTMP_PUBLISH_HOST
    def url(self, nonce=None):
        url = "rtmp://%s/%s/%s" % (self.rtmp_publish_host, self.__hub__, self.__title__)
        if self.__security__ == "static":
            url += "?key=%s" % self.__key__
        elif self.__security__ == "dynamic":
            if nonce is None:
                nonce = str(int(time.time()*1000))
            url += "?nonce=%s" % nonce
            parsed = urlparse(url)
            data = "%s?%s" % (parsed.path, parsed.query)
            token = __hmac_sha1__(data, self.__key__)
            url += "&token=%s" % token
            pass
        else:
            return None
        return url



from setuptools import setup, find_packages

setup(
    name = 'pili',
    version = '0.0.6',
    keywords = ('pili'),
    description = 'SDK for pili',
    license = 'MIT License',
    install_requires = [],

    author = 'fanchenlin',
    author_email = 'fanchenlin@qiniu.com',

    packages = find_packages(),
    platforms = 'any',
)
